-- =========================== add_ip_address.adb ============================
-- 
--  Copyright 2001 by Craig Spannring
-- 
--  Def:   Add an IP alias to an interface.
--         This program only works with Windows 2000 or latter.
-- 
-- ===========================================================================

with Ada.Text_IO; use Ada.Text_IO;
with Ada.Command_Line;
with Ada.Exceptions;


with Iphlpapi.thin;    use Iphlpapi.Thin; 
with Iphlpapi.Helpers; use Iphlpapi.Helpers; 

procedure Add_Ip_Address is 
   rcs_body_revision_id: constant string := "$Header: /home/cts/PersonalRepository/iphlpapi/add_ip_address.adb,v 1.1.1.1.2.4 2001/03/24 02:14:57 cts Exp $";
   
   package Cl renames Ada.Command_Line;
   
   use type DWORD;
   
   procedure Usage is 
   begin
      Put_Line("Usage- " & Cl.Command_Name 
               & " ip_address [netmask [interface]]");
      New_Line;
      Put_Line("ip_address and netmask must a decimal dotted quad format");
      Put_Line("interface is the number of the interface you wish to change");
   end Usage;
   
   
   Rc:        Dword;
   Context:   aliased ULONG;
   Instance:  aliased ULONG;
   Ip:        Ipaddr; 
   Mask:      Ipaddr  := String_To_Ip_Address("255.255.255.254"); 
   Interface: DWORD   := 2;
begin
   if Cl.Argument_Count < 1 or Cl.argument_Count > 3 then 
      Usage;
      Cl.Set_Exit_Status(Cl.Failure);
      return;
   end if;
   
   if Cl.Argument_Count = 3 then 
      Interface := DWORD'Value(Cl.Argument(3));
   end if;
   
   if Cl.Argument_Count >= 2 then 
      Mask := String_To_Ip_Address(Cl.Argument(2));
   end if;
   
   Ip := String_To_Ip_Address(Cl.Argument(1));
               
   Rc := AddIPAddress(Ip,
                      Mask,
                      interface, 
                      Context'Unchecked_Access,
                      Instance'Unchecked_Access);
   if Rc = 0 then 
      Put_Line("Context is " & ULONG'Image(Context));
   else
      Cl.Set_Exit_Status(Cl.Failure);
      Put_Line("Error.  rc = " & DWORD'Image(Rc));
   end if;
exception
   when Exc: others => 
      Put_Line(Ada.Exceptions.Exception_Message(Exc));
      Usage;
      Cl.Set_Exit_Status(Cl.Failure);
end Add_Ip_Address;
