-- ========================= create_ip_net_entry.adb =========================
-- 
--  Copyright 2001 by Craig Spannring
-- 
--  Def:   Create a new entry in the local computers ARP table.
-- 
-- ===========================================================================


with Ada.Text_IO; use Ada.Text_IO;
with Ada.Command_Line;
with Ada.Exceptions;


with Iphlpapi.thin;    use Iphlpapi.Thin; 
with Iphlpapi.Helpers; use Iphlpapi.Helpers; 

procedure Create_Ip_Net_Entry is 
   rcs_body_revision_id: constant string := "$Header: /home/cts/PersonalRepository/iphlpapi/Attic/create_ip_net_entry.adb,v 1.1.2.2 2001/03/24 04:32:51 cts Exp $";
   
   package Cl renames Ada.Command_Line;
   
   use type DWORD;
   
   procedure Usage is 
   begin
      Put_Line("Usage- " & Cl.Command_Name & " IP Physical [interface_index]");
      New_Line;
      Put_Line("  IP is the decimal dotted quad IP address of the host entry");
      Put_Line("  Physical is the MAC address of the host");
      Put_Line("  interface is the index of the interface you wish to change");
   end Usage;
   
   
   Rc:        Dword;
   arp:       aliased MIB_IPNETROW;
   ip:        Ipaddr; 
   phys:      PHYSADDR_BUFFER;
   Interface: DWORD   := 2;
begin
   if Cl.Argument_Count < 2 or Cl.Argument_Count > 3 then 
      Usage;
      Cl.Set_Exit_Status(Cl.Failure);
      return;
   end if;
   ip      := String_To_Ip_Address(Cl.Argument(1));
   Phys    := String_To_Ethernet_Address(Cl.Argument(2));
   if Cl.Argument_Count >= 3 then 
      Interface := DWORD'Value(Cl.Argument(3));
   end if;
   
   arp := (dwIndex        => Interface,
           dwPhysAddrLen  => 6,
           bPhysAddr      => Phys,
           dwAddr         => IP,
           dwType         => 3);
   
   Rc := CreateIpNetEntry(Arp'Unchecked_Access);
   
   if Rc /= 0 then 
      Cl.Set_Exit_Status(Cl.Failure);
      Put_Line("Error.  rc = " & DWORD'Image(Rc));
   end if;
exception
   when Exc: others => 
      Put_Line(Ada.Exceptions.Exception_Message(Exc));
      Usage;
      Cl.Set_Exit_Status(Cl.Failure);
end Create_Ip_Net_Entry;
