-- ======================= create_proxy_arp_entry.adb ========================
-- 
--  Copyright 2001 by Craig Spannring
-- 
--  Def:   Create a proxy ARP entry on the local computer.
-- 
-- ===========================================================================




with Ada.Text_IO; use Ada.Text_IO;
with Ada.Command_Line;
with Ada.Exceptions;


with Iphlpapi.thin;    use Iphlpapi.Thin; 
with Iphlpapi.Helpers; use Iphlpapi.Helpers; 

procedure Create_Proxy_Arp_Entry is 
   rcs_body_revision_id: constant string := "$Header: /home/cts/PersonalRepository/iphlpapi/Attic/create_proxy_arp_entry.adb,v 1.1.2.2 2001/03/24 04:32:51 cts Exp $";
   
   package Cl renames Ada.Command_Line;
   
   use type DWORD;
   
   procedure Usage is 
   begin
      Put_Line("Usage- " & Cl.Command_Name & " addr mask [interface_index]");
      New_Line;
      Put_Line("  IP is the decimal dotted quad IP address of the host entry");
      Put_Line("  Physical is the MAC address of the host");
      Put_Line("  interface is the index of the interface you wish to change");
   end Usage;
   
   
   Rc:        Dword;
   ip:        Ipaddr; 
   mask:      Ipaddr := String_To_Ip_Address("255.255.255.0"); 
   Interface: DWORD   := 2;
begin
   if Cl.Argument_Count < 1 or Cl.Argument_Count > 3 then 
      Usage;
      Cl.Set_Exit_Status(Cl.Failure);
      return;
   end if;
   ip      := String_To_Ip_Address(Cl.Argument(1));
   if Cl.Argument_Count > 2 then 
      mask    := String_To_Ip_Address(Cl.Argument(2));
   end if;
   if Cl.Argument_Count >= 3 then 
      Interface := DWORD'Value(Cl.Argument(3));
   end if;
   
   Rc := CreateProxyArpEntry(Ip, Mask, Interface);
   
   if Rc /= 0 then 
      Cl.Set_Exit_Status(Cl.Failure);
      Put_Line("Error.  rc = " & DWORD'Image(Rc));
   end if;
exception
   when Exc: others => 
      Put_Line(Ada.Exceptions.Exception_Message(Exc));
      Usage;
      Cl.Set_Exit_Status(Cl.Failure);
end Create_Proxy_Arp_Entry;
