-- ============================ get_if_table.adb =============================
-- 
--  Copyright 2001 by Craig Spannring
-- 
--  Def:   Print information about all the interfaces on the local computer.
-- 
-- ===========================================================================


with Ada.Text_IO; use Ada.Text_IO;
with Ada.Wide_Text_Io; 
with Ada.Command_Line;
with Ada.Exceptions;

with Interfaces.C;


with Iphlpapi.thin;    use Iphlpapi.Thin; 
with Iphlpapi.Helpers; use Iphlpapi.Helpers; 

procedure Get_If_Table is 
   rcs_body_revision_id: constant string := "$Header: /home/cts/PersonalRepository/iphlpapi/Attic/get_if_table.adb,v 1.1.2.1 2001/03/24 04:32:52 cts Exp $";
   
   package Ic   renames Interfaces.C;
   package Wtio renames Ada.Wide_Text_Io;
   package Cl renames Ada.Command_Line;
   
   use type DWORD;
   
   procedure Usage is 
   begin
      Put_Line("Usage- " & Cl.Command_Name & " [interface_index]");
      New_Line;
      Put_Line("  interface is the index of the interface you wish to view");
   end Usage;
   
   function cvt(Item     : in Ic.wchar_array;
                Trim_Nul : in Boolean := True) return Wide_String
     renames Interfaces.c.To_Ada;
   
   Rc:        DWORD;
   Table:     aliased MIB_IFTABLE;
   Size:      aliased DWORD := Table'Size/Ic.CHAR_BIT;
begin
   -- 
   -- Retrieve the information about all of the interfaces.  Place the
   -- information into the structure 'Table'.
   -- 
   Rc := GetIfTable(Table'Unchecked_Access, 
                    Size'Unchecked_Access,
                    1);
   
   
   if Rc = 0 then 
      --
      -- The first field in the MIB_IFTABLE structure is dwNumEntries.
      -- It contains a count of how many interfaces are in the local
      -- computer.  
      -- 
      for I in 0..table.dwNumEntries-1 loop
         declare
            --             
            -- The second field in the MIB_IFTABLE structure is an
            -- array of MIB_IFROW structures..  There is one MIB_IFROW
            -- for each interface in the machine.  
            --             
            Info: MIB_IFROW renames Table.Table(Integer(I));
         begin
            --
            -- Now print all the information for this element in the
            -- array.
            -- 
            Wtio.Put(Cvt(Info.wszName));
            New_Line;
            Put_Line("   Interface:            " & DWORD'Image(Info.dwIndex));
            Put_Line("   Type:                 " & DWORD'Image(Info.dwType));
            Put_Line("   MTU:                  " & DWORD'Image(Info.dwMtu));
            Put_Line("   Speed:                " & DWORD'Image(Info.dwSpeed));
            Put_Line("   MAC length:           " & DWORD'Image(Info.dwPhysAddrLen));
            if Info.dwPhysAddrLen > 0 then 
               Put_Line("   MAC:                  " & " " & Physical_Address_To_String(Info.bPhysAddr,
                                                                                       Integer(Info.dwPhysAddrLen)));
            end if;
            Put_line("   Admin Status:         " & DWORD'Image(Info.dwAdminStatus));
            Put("   Oper Status:           ");
            case Info.dwOperStatus is 
               when MIB_IF_OPER_STATUS_NON_OPERATIONAL => 
                  Put_Line("NON_OPERATIONAL");
               when MIB_IF_OPER_STATUS_UNREACHABLE => 
                  Put_Line("UNREACHABLE");
               when MIB_IF_OPER_STATUS_DISCONNECTED => 
                  Put_Line("DISCONNECTED");
               when MIB_IF_OPER_STATUS_CONNECTING => 
                  Put_Line("CONNECTING");
               when MIB_IF_OPER_STATUS_CONNECTED => 
                  Put_Line("CONNECTED");
               when MIB_IF_OPER_STATUS_OPERATIONAL => 
                  Put_Line("OPERATIONAL");
               when others => 
                  Put_Line("UNKNOWN");
            end case;
            Put_Line("   Last Change:          " & DWORD'Image(Info.dwLastChange));
            Put_line("   Bytes In:             " & DWORD'Image(Info.dwInOctets));
            Put_Line("   Unicast Pkts In:      " & DWORD'Image(Info.dwInUcastPkts));
            Put_Line("   non-Unicast Pkts In:  " & DWORD'Image(Info.dwInNUcastPkts));
            Put_Line("   In Pkts discarded:    " & DWORD'Image(Info.dwInDiscards));
            Put_Line("   Input Pkt Errors:     " & DWORD'Image(Info.dwInErrors));
            Put_Line("   Unknown Pkts:         " & DWORD'Image(Info.dwInUnknownProtos));
            Put_Line("   Bytes Out:            " & DWORD'Image(Info.dwOutOctets));
            Put_Line("   Unicast Pkts Out:     " & DWORD'Image(Info.dwOutUcastPkts)); 
            Put_Line("   non-Unicast Pkts Out: " & DWORD'Image(Info.dwOutNUcastPkts));
            Put_Line("   Out Pkts discarded:   " & DWORD'Image(Info.dwOutDiscards));
            Put_Line("   Output Pkt Errors:    " & DWORD'Image(Info.dwOutErrors));
            Put_Line("   Output Queue Length:  " & DWORD'Image(Info.dwOutQLen));
            Put_Line("   Description Length:   " & DWORD'Image(Info.dwDescrLen));
            Put_line("   Description:           " & To_String(Info.bDescr, Integer(Info.dwDescrLen)));
         end;
      end loop;
   else
      Cl.Set_Exit_Status(Cl.Failure);
      Put_Line("Error.  rc = " & DWORD'Image(Rc));
   end if;
exception
   when Exc: others => 
      Put_Line(Ada.Exceptions.Exception_Message(Exc));
      Usage;
      Cl.Set_Exit_Status(Cl.Failure);
end Get_If_Table;
