-- ========================== iphlpapi-helpers.ads ===========================
-- 
--  Copyright 2001 by Craig Spannring
-- 
--  Def:   helper functions for using iphlpapi.thin
-- 
-- ===========================================================================

with Ada.Characters.Handling;
with Ada.Strings;
with Ada.Strings.Fixed;
with Ada.Strings.Maps;



package body Iphlpapi.Helpers is
   rcs_body_revision_id: constant string := "$Header: /home/cts/PersonalRepository/iphlpapi/iphlpapi-helpers.adb,v 1.1.1.1.2.3 2001/03/24 04:32:53 cts Exp $";
      
   
   Not_Implemented:  exception;
   
   
   hex_digits: constant array (BYTE range 0..15) of Character
     :=  ('0', '1', '2', '3', '4', 
          '5', '6', '7', '8', '9', 
          'A', 'B', 'C', 'D', 'E', 'F');
   
   
   function Net_To_Host_Dword (V: DWORD) return DWORD is
   begin
      raise Not_Implemented;
      return 0;
   end Net_To_Host_Dword;
   
   
   function Host_To_Net_Dword (V: DWORD) return DWORD is
      Result: DWORD;
   begin
      raise Not_Implemented;
      return 0;
   end Host_To_Net_Dword;

   
   function Ip_Address_To_String (Addr: DWORD) return String is
      use Ada.Strings;
      use Ada.Strings.Fixed;
      use type DWORD;
   begin
      return (Trim(DWORD'Image((Addr/(256**0) mod 256)), Both)
              & "." & Trim(DWORD'Image((Addr/(256**1) mod 256)), Both)
              & "." & Trim(DWORD'Image((Addr/(256**2) mod 256)), Both)
              & "." & Trim(DWORD'Image((Addr/(256**3) mod 256)), Both)
              );
   end Ip_Address_To_String;
   

      
   function String_To_Ip_Address(Addr: String) return DWORD is 
      use Ada.Strings;
      use Ada.Strings.Fixed;
      use type DWORD;
      
      Result:  DWORD := 0;
      Tmp:     DWORD := 0;
      Str:     String := Trim(Addr, both);
      type Parts is range 0..3;
      Portion: Parts := 0;
   begin
      if Str(Str'First)='.' then 
         raise Constraint_Error;
      else
         for I in Str'Range loop
            if Str(I) /= '.' then
               Tmp := (Tmp * 10) + DWORD'Value(Str(I..I));
               if Tmp > 255 then 
                  raise Constraint_Error;
               end if;
            end if;
            
            if Str(I) = '.' or I=Str'Last then 
               Tmp     := Tmp * (256**Integer(Portion));
               Result  := Result + Tmp;
               Tmp     := 0;
               if I < Str'Last then 
                  Portion := Portion + 1;
               end if;
               if (I=Str'Last and Portion/=Parts'Last) then 
                  raise Constraint_Error;
               end if;
               if I/=Str'Last and then Str(I+1)='.' then 
                  raise Constraint_Error;
               end if;
            end if;
            end loop;
      end if;
      return Result;
   exception
      when others => 
         raise Invalid_Address;
   end String_To_Ip_Address;
   
   
   function Physical_Address_to_string(Addr:     PHYSADDR_BUFFER;
                                       Addr_Len: Positive := PHYSADDR_BUFFER'Length)
     return String is
      use Ada.Characters.Handling;
      function To_hex_string(V: Byte) return String is 
         use type BYTE;
      begin
         return To_Lower("" & hex_digits(V / 16) & hex_digits(V mod 16));
      end To_Hex_String;
      S:    string := To_Hex_String(Addr(Addr'First));
      Tmp:  PHYSADDR_BUFFER;
   begin
      if Addr_Len = 0 or Addr_Len > PHYSADDR_BUFFER'Length then 
         return "";
      elsif Addr_Len = 1 then 
         return S;
      else 
         Tmp(Tmp'First..Tmp'Last-1) := Addr(Addr'First+1..Addr'Last);
         return S & "-" & Physical_Address_To_String(Tmp, Addr_Len-1);
      end if;
   end Physical_Address_To_String;
   
   

   function String_To_Ethernet_Address(Addr: String) return PHYSADDR_BUFFER is
      use Ada.Characters.Handling;
      use Ada.Strings;
      use Ada.Strings.Fixed;
      use Ada.Strings.Maps;
      
      function To_BYTE(S: String) return BYTE is 
         Result:  integer := 0;
      begin
         if S'Length=0 then 
            raise Invalid_Address;
         else
            for I in S'Range loop
               if S(I) >= 'A' and S(I) <= 'F' then 
                  Result := Result*16 + 10+(Character'Pos(S(I)) - Character'Pos('A'));
               else
                  Result := Result*16 + Character'Pos(S(I)) - Character'Pos('0');
               end if;
               if Result > Integer(BYTE'Last) then 
                  raise Invalid_Address;
               end if;
            end loop;

            return BYTE(Result);
         end if;
      end To_BYTE;
                  
      Result: PHYSADDR_BUFFER := (others => 0);
      Index:  Natural := PHYSADDR_BUFFER'First;
      Digit:  constant Character_Set := To_Set("0123456789ABCDEF");
      Str:    String := To_Upper(Trim(Addr, both));
      first:  Positive := Str'First;
      Last:   Natural  := 0; 
      function Need_More_Data return Boolean is 
      begin
         return Index < Result'First+6;
      end Need_More_Data;
   begin
      loop
         if not Is_In(Str(First), digit) then 
            raise Invalid_Address;
         end if;
         if First > Str'Last then 
            raise Invalid_Address;
         end if;
         
         Find_Token(Source   => Str(First..Str'Last),
                    Set      => digit,
                    Test     => inside, 
                    First    => first, 
                    Last     => last);
         if Last=0 and Need_More_Data then 
            raise Invalid_Address;
         end if;
         exit when Last = 0;
         
         Result(Index) := to_BYTE(Str(First..Last));
         Index         := Index + 1;
         
         exit when Last = Str'Last;
         
         
         if Str(Last+1) /= '-' and Str(Last+1) /= ':' then 
            raise Invalid_Address;
         end if;
         
         First := Last + 2;
      end loop;
      
      if Need_More_Data then 
         raise Invalid_Address;
      end if;
      
      return Result;
   exception
      when others => 
         raise Invalid_Address;
   end String_To_Ethernet_Address;
   
   
   function Ethernet_Address_to_string(Addr: PHYSADDR_BUFFER) return String is
   begin
      return Physical_Address_To_String(Addr, 6);
   end Ethernet_Address_To_String;
   
   
   function To_String(Descr:   MIB_IFROW_If_Descr;
                      Length:  natural) return String 
   is
      use type Ic.Size_T;
      use Ada.Characters.Handling;
      Last:    constant Ic.size_t := Ic.size_t'min(Descr'Last, 
                                                   (Descr'First+Ic.Size_T(Length)-1));
      Result:  String(Natural(Descr'First+1)..Natural(Last+1)) := (others => '@');
   begin
      for I in Descr'First..Last loop
         if Is_Graphic(Ic.To_Ada(Descr(Ic.Size_T(I)))) then 
            Result(Integer(I+1)) := Ic.To_Ada(Descr(Ic.Size_T(I)));
         else
            Result(Integer(I+1)) := '~';
         end if;
      end loop;
      return Result;
   end To_String;
   
end Iphlpapi.Helpers;

