-- ============================ iphlpapi-thin.ads ============================
-- 
--  Copyright 2001 by Craig Spannring
-- 
--  Def:   thin binding to Microsoft's IP Helper SDK
-- 
--         The C API is documented on the MSDN subscription disks
--         and the Microsoft's developer web site.  As of March 22, 2001
--         you can view the documentation on-line at-
-- 
--            http://msdn.microsoft.com/library/psdk/rras/iphpport_7vz9.htm
--
--         The link location is subject to change.
-- 
-- ===========================================================================



with Interfaces.C;

with Win32.WinNT;


package Iphlpapi.thin is 
   rcs_spec_revision_id: constant string := "$Header: /home/cts/PersonalRepository/iphlpapi/Attic/iphlpapi-thin.ads,v 1.1.2.4 2001/03/24 04:32:53 cts Exp $";
   
   package Ic   renames Interfaces.C;
   
   use type Ic.Size_T;
   
   
   subtype BYTE  is Ic.Unsigned_Char;
   subtype UINT is Ic.Unsigned;
   subtype ULONG is Ic.Unsigned_Long;
   type ULONG_PTR is access all ULONG;
   subtype DWORD is Ic.Unsigned_Long;
   subtype IPAddr is Ic.Unsigned_Long;
   subtype IPMask is Ic.Unsigned_Long;
   subtype WCHAR is Ic.Wchar_T;
   type WSTR is access WCHAR;
   
   
   
   MAX_INTERFACE_NAME_LEN:   constant := 256;
   MAX_TRANSPORT_NAME_LEN:   constant := 40;
   MAX_MEDIA_NAME:           constant := 16;
   MAX_PORT_NAME:            constant := 16;
   MAX_DEVICE_NAME:          constant := 128;
   MAX_PHONE_NUMBER_LEN:     constant := 128;
   MAX_DEVICETYPE_NAME:      constant := 16;
   
   
   --
   -- The following constant correspong with the MIB-II protocol
   -- Hopefully those are the same values the iphlpapi.dll wants.
   --
   PROTO_IP_OTHER:      constant := 1;
   PROTO_IP_LOCAL:      constant := 2;
   PROTO_IP_NETMGMT:    constant := 3;
   PROTO_IP_ICMP:       constant := 4;
   PROTO_IP_EGP:        constant := 5;
   PROTO_IP_GGP:        constant := 6;
   PROTO_IP_HELLO:      constant := 7;
   PROTO_IP_RIP:        constant := 8;
   PROTO_IP_IS_IS:      constant := 9;
   PROTO_IP_ES_IS:      constant := 10;
   PROTO_IP_CISCO:      constant := 11;
   PROTO_IP_BBN:        constant := 12;
   PROTO_IP_OSPF:       constant := 13;
   PROTO_IP_BGP:        constant := 14;
   
   MAXLEN_IFDESCR:      constant := 256;
   MAXLEN_PHYSADDR:     constant := 8;
      
   MIB_IF_OPER_STATUS_NON_OPERATIONAL:      constant := 0;
   MIB_IF_OPER_STATUS_UNREACHABLE:          constant := 1;
   MIB_IF_OPER_STATUS_DISCONNECTED:         constant := 2;
   MIB_IF_OPER_STATUS_CONNECTING:           constant := 3;
   MIB_IF_OPER_STATUS_CONNECTED:            constant := 4;
   MIB_IF_OPER_STATUS_OPERATIONAL:          constant := 5;
   
   
   
   type PHYSADDR_BUFFER is array(0..(MAXLEN_PHYSADDR-1)) of BYTE;
   pragma Convention(C, PHYSADDR_BUFFER);
   
   type MIB_IFROW_If_Descr is new Ic.Char_Array(0..(MAXLEN_IFDESCR-1));

   subtype MIBIFROW_If_Name is Ic.Wchar_Array(0..(MAX_INTERFACE_NAME_LEN-1));
   
   
   type MIB_IFROW is record
      wszName:            Ic.Wchar_Array(0..(MAX_INTERFACE_NAME_LEN-1)); -- MIBIFROW_If_Name;
      dwIndex:            DWORD;
      dwType:             DWORD;
      dwMtu:              DWORD;
      dwSpeed:            DWORD;
      dwPhysAddrLen:      DWORD;
      BPhysAddr:          PHYSADDR_BUFFER;
      dwAdminStatus:      DWORD;
      dwOperStatus:       DWORD;
      dwLastChange:       DWORD;
      dwInOctets:         DWORD;
      dwInUcastPkts:      DWORD;
      dwInNUcastPkts:     DWORD;
      dwInDiscards:       DWORD;
      dwInErrors:         DWORD;
      dwInUnknownProtos:  DWORD;
      dwOutOctets:        DWORD;
      dwOutUcastPkts:     DWORD;
      dwOutNUcastPkts:    DWORD;
      dwOutDiscards:      DWORD;
      dwOutErrors:        DWORD;
      dwOutQLen:          DWORD;
      dwDescrLen:         DWORD;
      bDescr:             MIB_IFROW_If_Descr;
   end record;
   pragma Convention(C, MIB_IFROW);

   type MIB_IFTABLE_Array is array(Natural range <>) of MIB_IFROW;
   pragma Convention(C, MIB_IFTABLE_Array);
   
   type MIB_IFTABLE is record
      dwNumEntries:  DWORD;                     -- number of entries in table
      Table:         MIB_IFTABLE_Array(0..255); -- array of interface entries
   end record;
   pragma Convention(C, MIB_IFTABLE);
      
  
  
   type MIB_IPNETROW is record
      dwIndex:        DWORD;
      dwPhysAddrLen:  DWORD;
      bPhysAddr:      PHYSADDR_BUFFER;
      dwAddr:         DWORD;
      dwType:         DWORD;
   end record;
   pragma Convention(C, MIB_IPNETROW);
    
      
      
      
   type MIB_IPFORWARDROW is record
      dwForwardDest:       DWORD;  -- IP addr of destination
      dwForwardMask:       DWORD;  -- subnetwork mask of destination
      dwForwardPolicy:     DWORD;  -- conditions for multi-path route
      dwForwardNextHop:    DWORD;  -- IP address of next hop
      dwForwardIfIndex:    DWORD;  -- index of interface
      dwForwardType:       DWORD;  -- route type
      dwForwardProto:      DWORD;  -- protocol that generated route
      dwForwardAge:        DWORD;  -- age of route
      dwForwardNextHopAS:  DWORD;  -- autonomous system number 
                                   -- of next hop
      dwForwardMetric1:    DWORD;  -- protocol-specific metric 
      dwForwardMetric2:    DWORD;  -- protocol-specific metric 
      dwForwardMetric3:    DWORD;  -- protocol-specific metric 
      dwForwardMetric4:    DWORD;  -- protocol-specific metric 
      dwForwardMetric5:    DWORD;  -- protocol-specific metric 
   end record;
   pragma Convention(C, MIB_IPFORWARDROW);
  
  
   type MIB_IPADDRROW is record
      dwAddr:       DWORD             := 1; -- IP address
      dwIndex:      DWORD             := 1; -- interface index
      dwMask:       DWORD             := 1; -- subnet mask
      dwBCastAddr:  DWORD             := 1; -- broadcast address 
      dwReasmSize:  DWORD             := 1; -- rassembly size 
      Unused1:      Ic.Unsigned_Short := 1; -- not currently used 
      Unused2:      Ic.Unsigned_Short := 1;    
   end record;
   pragma Convention(C, MIB_IPADDRROW);

   type MIB_IPADDRTABLE_Array is array(Natural range <>) of MIB_IPADDRROW;
   pragma Convention(C, MIB_IPADDRTABLE_Array);
   
   type MIB_IPADDRTABLE  is record 
      dwNumEntries: DWORD := 256;
      Table:        MIB_IPADDRTABLE_Array(0..255);   -- array of IP address Entries
   end record;
   pragma Convention(C, MIB_IPADDRTABLE);
   pragma Volatile(MIB_IPADDRTABLE);
   
   
   type MIB_IPSTATS is record
     dwForwarding:       DWORD;  -- IP forwarding enabled or disabled
     dwDefaultTTL:       DWORD;  -- default time-to-live
     dwInReceives:       DWORD;  -- datagrams received
     dwInHdrErrors:      DWORD;  -- received header errors
     dwInAddrErrors:     DWORD;  -- received address errors
     dwForwDatagrams:    DWORD;  -- datagrams forwarded
     dwInUnknownProtos:  DWORD;  -- datagrams with unknown protocol
     dwInDiscards:       DWORD;  -- received datagrams discarded
     dwInDelivers:       DWORD;  -- received datagrams delivered
     dwOutRequests:      DWORD;  --
     dwRoutingDiscards:  DWORD;  --
     dwOutDiscards:      DWORD;  -- sent datagrams discarded
     dwOutNoRoutes:      DWORD;  -- datagrams for which no route exists
     dwReasmTimeout:     DWORD;  -- datagrams for which all 
                                 -- frags did not arrive
     dwReasmReqds:       DWORD;  -- datagrams requiring reassembly
     dwReasmOks:         DWORD;  -- successful reassemblies
     dwReasmFails:       DWORD;  -- failed reassemblies
     dwFragOks:          DWORD;  -- successful fragmentations
     dwFragFails:        DWORD;  -- failed fragmentations
     dwFragCreates:      DWORD;  -- datagrams fragmented
     dwNumIf:            DWORD;  -- number of interfaces on computer
     dwNumAddr:          DWORD;  -- number of IP address on computer
     dwNumRoutes:        DWORD;  -- number of routes in routing table
   end record;
   pragma Convention(C, MIB_IPSTATS);
   
   
   type OVERLAPPED is record
      Internal:       ULONG_PTR; 
      InternalHigh:   ULONG_PTR; 
      Offset:         DWORD; 
      OffsetHigh:     DWORD; 
      HEvent:         Win32.Winnt.Handle; 
   end record; 
   pragma Convention (C, OVERLAPPED);

   
            
   
   ---
   -- Add the specified IP address to the specified adapter.
   -- 
   -- Works on Win2K
   -- 
   function AddIPAddress(Address:     IPAddr;
                         Mask:        IPMask;
                         IfIndex:     DWORD;
                         NTEContext:  access ULONG;
                         NTEInstance: access ULONG) 
     return DWORD;
   pragma Import(C, AddIPAddress, "AddIPAddress");
   
   
   ---
   -- Create a route in the local computer's IP routing table
   -- 
   -- Works on NT 4.0 SP4 or later, Win2K, and Windows 98
   -- 
   function CreateIpForwardEntry(PRoute:  access MIB_IPFORWARDROW )
     return DWORD;
   pragma Import(C, CreateIpForwardEntry, "CreateIpForwardEntry");
   
   
   ---
   -- Create an ARP entry in the ARP table on the local computer.
   --
   -- Works on NT 4.0 SP4 or later, Win2K, and Windows 98
   -- 
   function CreateIpNetEntry(pArpEntry: access MIB_IPNETROW) return DWORD;
   pragma Import(C, CreateIpNetEntry, "CreateIpNetEntry");
     
   
   ---
   -- Create a proxy arp entry on the local computer
   -- 
   -- Works on Win2K
   --
   function CreateProxyArpEntry(dwAddress:   DWORD;
                                dwMask:      DWORD;
                                dwIfIndex:   DWORD) return DWORD;
   pragma Import(C, CreateProxyArpEntry, "CreateProxyArpEntry");
   
   
   ---
   -- Delete an IP address that was created using AddIPAddress.
   -- 
   -- Works on Win2K
   --
   function DeleteIPAddress(NTEContext:  ULONG) return DWORD;
   pragma Import(C, DeleteIPAddress, "DeleteIPAddress");
   
   
   ---
   -- Delete an existing route in the local computer's IP
   -- routing table
   --
   --  Works on NT 4.0 SP4 or later, Win2K, and Windows 98
   -- 
   function DeleteIpForwardEntry(pRoute:  access MIB_IPFORWARDROW) return DWORD;
   pragma Import (C, DeleteIpForwardEntry, "DeleteIpForwardEntry");
   
   
   
   ---
   -- Delete an ARP entry from the ARP table on the local computer.
   -- 
   --  Works on NT 4.0 SP4 or later, Win2K, and Windows 98
   -- 
   function DeleteIpNetEntry(pArpEntry:  access MIB_IPNETROW) return DWORD;
   pragma Import (C, DeleteIpNetEntry, "DeleteIpNetEntry");
   
   
   
   ---
   -- Delete the specified proxy ARP entry on the local computer.
   -- 
   -- Works on Win2K
   --
   function DeleteProxyArpEntry(dwAddress:   DWORD;
                                dwMask:      DWORD;
                                dwIfIndex:   DWORD) return DWORD;
   pragma Import (C, DeleteProxyArpEntry, "DeleteProxyArpEntry");
   
   
   
   ---
   -- Turn on IP forwarding on the local computer.
   -- 
   function EnableRouter(pHandle:     access Win32.Winnt.Handle;
                         pOverlapped: access OVERLAPPED) return DWORD;
   pragma Import (C, EnableRouter, "EnableRouter");
   
   
   
   ---
   -- Delete all ARP entries for the specified interface in the ARP
   -- table on the local machine.
   -- 
   -- Works on Win2K
   --
   function FlushIpNetTable(dwIfIndex:  DWORD) return DWORD;
   pragma Import(C, FlushIpNetTable, "FlushIpNetTable");
   
   
   ---
   -- Lookup the index of an adapter by name.
   -- 
   -- Works on Win2K
   --
   function GetAdapterIndex(AdapterName:  WSTR;
                            IfIndex:      access ULONG) return DWORD;
   pragma Import(C, GetAdapterIndex, "GetAdapterIndex");
   
   
   ---
   -- Retrieve information for the specified adapter
   --
   --  Works on NT 4.0 SP4 or later, Win2K, and Windows 98
   -- 
   function GetIfEntry(pIfRow:  access MIB_IFROW) return DWORD;
   pragma Import(C, GetIfEntry, "GetIfEntry");
   
   
   ---
   -- Retrieve information for all the interfaces on the local
   -- computer. 
   -- 
   --  Works on NT 4.0 SP4 or later, Win2K, and Windows 98
   -- 
   function GetIfTable(pIfTable:    access MIB_IFTABLE;
                       pdwSize:     access DWORD;
                       bOrder:      DWORD) return DWORD;
   pragma Import(C, GetIfTable, "GetIfTable");
   
   
   ---
   -- Retrieve the IP statistics for the local computer.
   -- 
   -- Works on NT 4.0 SP4 or later, Win2K, and Windows 98
   -- 
   function GetIpStatistics(pStats:   access MIB_IPSTATS) return DWORD;
   pragma Import(C, GetIpStatistics, "GetIpStatistics");
   
   
   ---
   -- Retrieve the interface to IP mapping table.
   -- 
   -- Works on NT 4.0 SP4 or later, Win2K, and Windows 98
   -- 
   function GetIpAddrTable(lpIpAddrTable:   access MIB_IPADDRTABLE;
                           pdwSize:         access DWORD;
                           bOrder:          DWORD) return Dword;
   pragma Import(C, GetIpAddrTable, "GetIpAddrTable");
   
   
   ---
   -- Retrieve the number of interfaces on the local computer.
   -- 
   -- Works on NT 4.0 SP4 or later, Win2K, and Windows 98
   -- 
   function GetNumberOfInterfaces(pdwNumIf:  access DWORD) return DWORD;
   pragma Import(C, GetNumberOfInterfaces, "GetNumberOfInterfaces");
   
   
   ---
   -- Set the default time-to-live value for the local computer.
   -- 
   -- Works on NT 4.0 SP4 or later, Win2K, and Windows 98
   -- 
   function SetIpTTL(nTTL:  UINT) return DWORD;
   pragma Import (C, SetIpTTL, "SetIpTTL");

end Iphlpapi.thin;

   
   
